/*******************************************************************************
* File Name: CapSense_Proximity_CSHL.h
* Version 2.10
*
* Description:
*  This file provides constants and parameter values for the High Level APIs
*  for CapSense CSD component.
*
* Note:
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#if !defined(CY_CAPSENSE_CSD_CSHL_CapSense_Proximity_H)
#define CY_CAPSENSE_CSD_CSHL_CapSense_Proximity_H

#include "CapSense_Proximity.h"


/***************************************
*   Condition compilation parameters
***************************************/

#define CapSense_Proximity_SIGNAL_SIZE                    (16u)
#define CapSense_Proximity_AUTO_RESET                     (0u)
#define CapSense_Proximity_RAW_FILTER_MASK                (8u)

/* Signal size definition */
#define CapSense_Proximity_SIGNAL_SIZE_UINT8              (8u)
#define CapSense_Proximity_SIGNAL_SIZE_UINT16             (16u)

/* Auto reset definition */
#define CapSense_Proximity_AUTO_RESET_DISABLE             (0u)
#define CapSense_Proximity_AUTO_RESET_ENABLE              (1u)

/* Mask for RAW and POS filters */
#define CapSense_Proximity_MEDIAN_FILTER                  (0x01u)
#define CapSense_Proximity_AVERAGING_FILTER               (0x02u)
#define CapSense_Proximity_IIR2_FILTER                    (0x04u)
#define CapSense_Proximity_IIR4_FILTER                    (0x08u)
#define CapSense_Proximity_JITTER_FILTER                  (0x10u)
#define CapSense_Proximity_IIR8_FILTER                    (0x20u)
#define CapSense_Proximity_IIR16_FILTER                   (0x40u)
#define CapSense_Proximity_RAW_FILTERS_ENABLED            (0x01u)
#define CapSense_Proximity_RAW_FILTERS_DISABLED           (0x00u)

/***************************************
*           API Constants
***************************************/

/* Widgets constants definition */
#define CapSense_Proximity_PROXIMITYSENSOR__PROX        (0u)

#define CapSense_Proximity_TOTAL_DIPLEXED_SLIDERS_COUNT        (0u)
#define CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT          (0u)
#define CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT          (0u)
#define CapSense_Proximity_TOTAL_TOUCH_PADS_COUNT              (0u)
#define CapSense_Proximity_TOTAL_TOUCH_PADS_BASIC_COUNT        (0u)
#define CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT       (0u)
#define CapSense_Proximity_TOTAL_BUTTONS_COUNT                 (1u)
#define CapSense_Proximity_TOTAL_MATRIX_BUTTONS_COUNT          (0u)
#define CapSense_Proximity_TOTAL_GENERICS_COUNT                (0u)

#define CapSense_Proximity_POS_FILTERS_MASK                    (0x0u)
#define CapSense_Proximity_LINEAR_SLIDERS_POS_FILTERS_MASK     (0x0u)
#define CapSense_Proximity_RADIAL_SLIDERS_POS_FILTERS_MASK     (0x0u)
#define CapSense_Proximity_TOUCH_PADS_POS_FILTERS_MASK         (0x0u)
#define CapSense_Proximity_TRACKPAD_GEST_POS_FILTERS_MASK      (0x0u)

#define CapSense_Proximity_UNUSED_DEBOUNCE_COUNTER_INDEX       (1u)

#define CapSense_Proximity_TOTAL_PROX_SENSORS_COUNT            (1u)

#define CapSense_Proximity_END_OF_SLIDERS_INDEX                (0u)
#define CapSense_Proximity_END_OF_TOUCH_PAD_INDEX              (0u)
#define CapSense_Proximity_END_OF_BUTTONS_INDEX                (0u)
#define CapSense_Proximity_END_OF_MATRIX_BUTTONS_INDEX         (0u)
#define CapSense_Proximity_END_OF_WIDGETS_INDEX                (1u)



#define CapSense_Proximity_TOTAL_SLIDERS_COUNT            ( CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT + \
                                                          CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT )

#define CapSense_Proximity_TOTAL_CENTROIDS_COUNT          ( CapSense_Proximity_TOTAL_SLIDERS_COUNT + \
                                                         (CapSense_Proximity_TOTAL_TOUCH_PADS_BASIC_COUNT * 2u) +\
                                                         (CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT * 4u))

#define CapSense_Proximity_TOTAL_CENTROIDS_BASIC_COUNT    ( CapSense_Proximity_TOTAL_SLIDERS_COUNT + \
                                                         (CapSense_Proximity_TOTAL_TOUCH_PADS_BASIC_COUNT * 2u))

#define CapSense_Proximity_TOTAL_CENTROID_AXES_COUNT      ( CapSense_Proximity_TOTAL_SLIDERS_COUNT + \
                                                         (CapSense_Proximity_TOTAL_TOUCH_PADS_BASIC_COUNT * 2u) +\
                                                         (CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT * 2u))

#define CapSense_Proximity_TOTAL_WIDGET_COUNT             ( CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT + \
                                                          CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT + \
                                                          CapSense_Proximity_TOTAL_TOUCH_PADS_COUNT + \
                                                          CapSense_Proximity_TOTAL_BUTTONS_COUNT + \
                                                          CapSense_Proximity_TOTAL_MATRIX_BUTTONS_COUNT )

#define CapSense_Proximity_ANY_POS_FILTER                 ( CapSense_Proximity_MEDIAN_FILTER | \
                                                          CapSense_Proximity_AVERAGING_FILTER | \
                                                          CapSense_Proximity_IIR2_FILTER | \
                                                          CapSense_Proximity_IIR4_FILTER | \
                                                          CapSense_Proximity_JITTER_FILTER )

#define CapSense_Proximity_IS_DIPLEX_SLIDER               ( CapSense_Proximity_TOTAL_DIPLEXED_SLIDERS_COUNT > 0u)

#define CapSense_Proximity_IS_NON_DIPLEX_SLIDER           ( (CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT - \
                                                           CapSense_Proximity_TOTAL_DIPLEXED_SLIDERS_COUNT) > 0u)
#define CapSense_Proximity_ADD_SLIDER_TYPE                ((CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT > 0u) ? \
                                                        ((CapSense_Proximity_TOTAL_TOUCH_PADS_COUNT > 0u) || \
                                                         (CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT > 0u)) : 0u)

#define CapSense_Proximity_TOTAL_PROX_SENSOR_COUNT        (CapSense_Proximity_TOTAL_PROX_SENSORS_COUNT)

#define CapSense_Proximity_WIDGETS_TBL_SIZE               ( CapSense_Proximity_TOTAL_WIDGET_COUNT + \
                                                          CapSense_Proximity_TOTAL_GENERICS_COUNT)

#define CapSense_Proximity_WIDGET_PARAM_TBL_SIZE          (CapSense_Proximity_TOTAL_BUTTONS_COUNT + \
                                                         CapSense_Proximity_TOTAL_SLIDERS_COUNT +\
                                                         CapSense_Proximity_TOTAL_TOUCH_PADS_BASIC_COUNT * 2u + \
                                                         CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT * 2u +\
                                                         CapSense_Proximity_TOTAL_MATRIX_BUTTONS_COUNT * 2u)

#define CapSense_Proximity_THRESHOLD_TBL_SIZE         (CapSense_Proximity_WIDGET_PARAM_TBL_SIZE)
#define CapSense_Proximity_DEBOUNCE_CNT_TBL_SIZE      (CapSense_Proximity_WIDGET_PARAM_TBL_SIZE)
#define CapSense_Proximity_RAW_DATA_INDEX_TBL_SIZE    (CapSense_Proximity_WIDGET_PARAM_TBL_SIZE +\
                                                     CapSense_Proximity_TOTAL_GENERICS_COUNT)

#define CapSense_Proximity_RES_MULT                   (256u)


#define CapSense_Proximity_NOT_WIDGET                     (0xFFFFFFFFu)

/*Types of centroids */
#define CapSense_Proximity_TYPE_RADIAL_SLIDER             (0x01u)
#define CapSense_Proximity_TYPE_LINEAR_SLIDER             (0x02u)
#define CapSense_Proximity_TYPE_GENERIC                   (0xFFu)

/* Defines if sensors or widgets are active */
#define CapSense_Proximity_SENSOR_IS_ACTIVE               (0x01u)
#define CapSense_Proximity_SENSOR_1_IS_ACTIVE             (0x01u)
#define CapSense_Proximity_SENSOR_2_IS_ACTIVE             (0x02u)
#define CapSense_Proximity_WIDGET_IS_ACTIVE               (0x01u)

/* Defines diplex type of Slider */
#define CapSense_Proximity_IS_DIPLEX                      (0x80u)

/* Defines fingers positions on Slider  */
#define CapSense_Proximity_POS_PREV                       (0u)
#define CapSense_Proximity_POS                            (1u)
#define CapSense_Proximity_POS_NEXT                       (2u)
#define CapSense_Proximity_CENTROID_ROUND_VALUE           (0x7F00u)
#define CapSense_Proximity_MAXIMUM_CENTROID               (0xFFu)

#define CapSense_Proximity_NEGATIVE_NOISE_THRESHOLD       (20u)
#define CapSense_Proximity_LOW_BASELINE_RESET             (5u)


/***************************************
*        Function Prototypes
***************************************/

void CapSense_Proximity_InitializeSensorBaseline(uint32 sensor);
void CapSense_Proximity_InitializeAllBaselines(void);
void CapSense_Proximity_InitializeEnabledBaselines(void);
void CapSense_Proximity_UpdateSensorBaseline(uint32 sensor);
void CapSense_Proximity_UpdateBaselineNoThreshold(uint32 sensor);
void CapSense_Proximity_UpdateEnabledBaselines(void);
void CapSense_Proximity_UpdateWidgetBaseline(uint32 widget);
uint16 CapSense_Proximity_GetBaselineData(uint32 sensor);
void CapSense_Proximity_SetBaselineData(uint32 sensor, uint16 data);
void CapSense_Proximity_BaseInit(uint32 sensor);

#if (CapSense_Proximity_IS_DIPLEX_SLIDER)
    uint8 CapSense_Proximity_FindMaximum(uint8 offset, uint8 count, uint16 fingerThreshold, const uint8 *diplex);
#else
    uint8 CapSense_Proximity_FindMaximum(uint8 offset, uint8 count, uint16 fingerThreshold);
#endif /* (CapSense_Proximity_IS_DIPLEX_SLIDER) */

#if (CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT > 0u)
    uint8 CapSense_Proximity_CalcCentroid(uint8 maximum, uint8 offset, \
                                    uint8 count, uint32 resolution, uint16 noiseThreshold);
#else
    uint8 CapSense_Proximity_CalcCentroid(uint8 maximum, uint8 offset, \
                                    uint8 count, uint16 resolution, uint16 noiseThreshold);
#endif /* (CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT > 0u) */


uint16 CapSense_Proximity_GetFingerThreshold(uint32 widget);
uint16 CapSense_Proximity_GetNoiseThreshold(uint32 widget);
uint16 CapSense_Proximity_GetFingerHysteresis(uint32 widget);
uint16 CapSense_Proximity_GetNegativeNoiseThreshold(uint32 widget);

#if(CapSense_Proximity_TUNING_METHOD != CapSense_Proximity__TUNING_NONE)
    void CapSense_Proximity_SetFingerThreshold(uint32 widget, uint16 value);
    void CapSense_Proximity_SetNoiseThreshold(uint32 widget, uint16 value);
    void CapSense_Proximity_SetFingerHysteresis(uint32 widget, uint16 value);
    void CapSense_Proximity_SetNegativeNoiseThreshold(uint32 widget, uint16 value);
    void CapSense_Proximity_SetDebounce(uint32 widget, uint8 value);
    void CapSense_Proximity_SetLowBaselineReset(uint32 sensor, uint8 value);
#endif /* (CapSense_Proximity_TUNING_METHOD != CapSense_Proximity__TUNING_NONE) */

uint16 CapSense_Proximity_GetDiffCountData(uint32 sensor);
void CapSense_Proximity_SetDiffCountData(uint32 sensor, uint16 value);

uint32 CapSense_Proximity_CheckIsSensorActive(uint32 sensor);
uint32 CapSense_Proximity_CheckIsWidgetActive(uint32 widget);
uint32 CapSense_Proximity_CheckIsAnyWidgetActive(void);
void CapSense_Proximity_EnableWidget(uint32 widget);
void CapSense_Proximity_DisableWidget(uint32 widget);
void CapSense_Proximity_EnableRawDataFilters(void);
void CapSense_Proximity_DisableRawDataFilters(void);

#if (CapSense_Proximity_TOTAL_MATRIX_BUTTONS_COUNT)
    uint32 CapSense_Proximity_GetMatrixButtonPos(uint32 widget, uint8* pos);
#endif /* (CapSense_Proximity_TOTAL_MATRIX_BUTTONS_COUNT) */

#if((CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT > 0u) || (CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT > 0u))
    uint16 CapSense_Proximity_GetCentroidPos(uint32 widget);
#endif /* ((CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT > 0u) || (CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT > 0u)) */
#if((CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT > 0u) || (CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT > 0u))
    uint16 CapSense_Proximity_GetRadialCentroidPos(uint32 widget);
#endif /* #if((CapSense_Proximity_TOTAL_RADIAL_SLIDERS_COUNT > 0u) || (CapSense_Proximity_TOTAL_LINEAR_SLIDERS_COUNT > 0u)) */
#if (CapSense_Proximity_TOTAL_TOUCH_PADS_COUNT)
    uint32 CapSense_Proximity_GetTouchCentroidPos(uint32 widget, uint16* pos);
#endif /* (CapSense_Proximity_TOTAL_TOUCH_PADS_COUNT) */

uint32 CapSense_Proximity_GetWidgetNumber(uint32 sensor);
uint8 CapSense_Proximity_GetLowBaselineReset(uint32 sensor);
uint8 CapSense_Proximity_GetDebounce(uint32 widget);

/* Filter function prototypes for High level APIs */

/* Median filter function prototype */
#if ( (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_MEDIAN_FILTER)) || \
      (0u != (CapSense_Proximity_POS_FILTERS_MASK & CapSense_Proximity_MEDIAN_FILTER)) || \
      ((CapSense_Proximity_TUNING_METHOD == CapSense_Proximity__TUNING_AUTO)) )
    uint16 CapSense_Proximity_MedianFilter(uint16 x1, uint16 x2, uint16 x3);
#endif /* ( (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_MEDIAN_FILTER)) || \
      (0u != (CapSense_Proximity_POS_FILTERS_MASK & CapSense_Proximity_MEDIAN_FILTER)) || \
      ((CapSense_Proximity_TUNING_METHOD == CapSense_Proximity__TUNING_AUTO)) ) */

/* Averaging filter function prototype */
#if ( (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_AVERAGING_FILTER)) || \
      (0u != (CapSense_Proximity_POS_FILTERS_MASK & CapSense_Proximity_AVERAGING_FILTER)) )
    uint16 CapSense_Proximity_AveragingFilter(uint16 x1, uint16 x2, uint16 x3);
#endif /* CapSense_Proximity_RAW_FILTER_MASK && CapSense_Proximity_POS_FILTERS_MASK */

/* IIR2Filter(1/2prev + 1/2cur) filter function prototype */
#if ( (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_IIR2_FILTER)) || \
      (0u != (CapSense_Proximity_POS_FILTERS_MASK & CapSense_Proximity_IIR2_FILTER)) )
    uint16 CapSense_Proximity_IIR2Filter(uint16 x1, uint16 x2);
#endif /* CapSense_Proximity_RAW_FILTER_MASK && CapSense_Proximity_POS_FILTERS_MASK */

/* IIR4Filter(3/4prev + 1/4cur) filter function prototype */
#if ( (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_IIR4_FILTER)) || \
      (0u != (CapSense_Proximity_POS_FILTERS_MASK & CapSense_Proximity_IIR4_FILTER)) )
    uint16 CapSense_Proximity_IIR4Filter(uint16 x1, uint16 x2);
#endif /* CapSense_Proximity_RAW_FILTER_MASK && CapSense_Proximity_POS_FILTERS_MASK */

/* IIR8Filter(7/8prev + 1/8cur) filter function prototype - RawCounts only */
#if (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_IIR8_FILTER))
    uint16 CapSense_Proximity_IIR8Filter(uint16 x1, uint16 x2);
#endif /* CapSense_Proximity_RAW_FILTER_MASK */

/* IIR16Filter(15/16prev + 1/16cur) filter function prototype - RawCounts only */
#if (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_IIR16_FILTER))
    uint16 CapSense_Proximity_IIR16Filter(uint16 x1, uint16 x2);
#endif /* CapSense_Proximity_RAW_FILTER_MASK */

/* JitterFilter filter function prototype */
#if ( (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_JITTER_FILTER)) || \
      (0u != (CapSense_Proximity_POS_FILTERS_MASK & CapSense_Proximity_JITTER_FILTER)) || \
      (0u != (CapSense_Proximity_TRACKPAD_GEST_POS_FILTERS_MASK & CapSense_Proximity_JITTER_FILTER)))
    uint16 CapSense_Proximity_JitterFilter(uint16 x1, uint16 x2);
#endif /* ( (0u != (CapSense_Proximity_RAW_FILTER_MASK & CapSense_Proximity_JITTER_FILTER)) || \
       *    (0u != (CapSense_Proximity_POS_FILTERS_MASK & CapSense_Proximity_JITTER_FILTER)) )
       *    (0u != (CapSense_Proximity_TRACKPAD_GEST_POS_FILTERS_MASK & CapSense_Proximity_JITTER_FILTER)) )
       */


/***************************************
*     Vars with External Linkage
***************************************/
extern uint16 CapSense_Proximity_sensorBaseline[CapSense_Proximity_TOTAL_SENSOR_COUNT];
extern uint8  CapSense_Proximity_sensorBaselineLow[CapSense_Proximity_TOTAL_SENSOR_COUNT];
extern uint16 CapSense_Proximity_sensorSignal[CapSense_Proximity_TOTAL_SENSOR_COUNT];
extern uint8  CapSense_Proximity_sensorOnMask[CapSense_Proximity_TOTAL_SENSOR_MASK];

extern uint8 CapSense_Proximity_lowBaselineResetCnt[CapSense_Proximity_TOTAL_SENSOR_COUNT];
extern uint8 CapSense_Proximity_lowBaselineReset[CapSense_Proximity_TOTAL_SENSOR_COUNT];

/* Generated by Customizer */
#if (CapSense_Proximity_TUNING_METHOD != CapSense_Proximity__TUNING_NONE)
    extern uint16 CapSense_Proximity_fingerThreshold[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
    extern uint16 CapSense_Proximity_noiseThreshold[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
    extern uint16 CapSense_Proximity_negativeNoiseThreshold[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
    extern uint16 CapSense_Proximity_hysteresis[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
    extern uint8  CapSense_Proximity_debounce[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
#else
    extern const uint16 CapSense_Proximity_fingerThreshold[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
    extern const uint16 CapSense_Proximity_noiseThreshold[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
    extern const uint16 CapSense_Proximity_hysteresis[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
    extern const uint8  CapSense_Proximity_debounce[CapSense_Proximity_WIDGET_PARAM_TBL_SIZE];
#endif /* (CapSense_Proximity_TUNING_METHOD != CapSense_Proximity__TUNING_NONE) */

extern const uint8 CapSense_Proximity_rawDataIndex[CapSense_Proximity_RAW_DATA_INDEX_TBL_SIZE];
extern const uint8 CapSense_Proximity_numberOfSensors[CapSense_Proximity_RAW_DATA_INDEX_TBL_SIZE];

#if (0u != CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT)
    #if (CapSense_Proximity_TUNING_METHOD != CapSense_Proximity__TUNING_NONE)
        extern uint8 CapSense_Proximity_posFiltersMask[CapSense_Proximity_TOTAL_CENTROID_AXES_COUNT];
    #else
        extern const uint8 CapSense_Proximity_posFiltersMask[CapSense_Proximity_TOTAL_CENTROID_AXES_COUNT];
    #endif /* (CapSense_Proximity_TUNING_METHOD != CapSense_Proximity__TUNING_NONE) */

    extern uint32 CapSense_Proximity_centroidMult[CapSense_Proximity_TOTAL_CENTROID_AXES_COUNT];
#endif /* (0u != CapSense_Proximity_TOTAL_TRACKPAD_GESTURES_COUNT) */

/***************************************
*        Obsolete Names
***************************************/
#define CapSense_Proximity_SensorRaw              CapSense_Proximity_sensorRaw
#define CapSense_Proximity_SensorEnableMask       CapSense_Proximity_sensorEnableMask
#define CapSense_Proximity_SensorBaseline         CapSense_Proximity_sensorBaseline
#define CapSense_Proximity_SensorBaselineLow      CapSense_Proximity_sensorBaselineLow
#define CapSense_Proximity_SensorSignal           CapSense_Proximity_sensorSignal
#define CapSense_Proximity_SensorOnMask           CapSense_Proximity_sensorOnMask
#define CapSense_Proximity_LowBaselineResetCnt    CapSense_Proximity_lowBaselineResetCnt

#endif /* CY_CAPSENSE_CSD_CSHL_CapSense_Proximity_H */

/* [] END OF FILE */
